@@ -1,29 +1,45 @@
 import logging
 from copy import deepcopy
 
+from allauth import app_settings as allauth_settings
 from allauth.account.models import EmailAddress
 from django import forms
 from django.apps import apps
+from django.conf import settings
 from django.contrib import admin, messages
+from django.contrib.admin.actions import delete_selected
+from django.contrib.admin.sites import NotRegistered
 from django.contrib.admin.utils import model_ngettext
+from django.contrib.auth import get_user_model
 from django.contrib.auth.admin import GroupAdmin as BaseGroupAdmin
 from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
 from django.contrib.auth.forms import UserChangeForm as BaseUserChangeForm
 from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm
+from django.core.exceptions import ValidationError
 from django.forms.models import BaseInlineFormSet
+from django.http import HttpResponseRedirect
 from django.template.response import TemplateResponse
+from django.urls import reverse
+from django.utils.translation import ngettext
 from django.utils.translation import ugettext_lazy as _
 from openwisp_utils.admin import UUIDAdmin
-from organizations.base_admin import (BaseOrganizationAdmin,
-                                      BaseOrganizationOwnerAdmin,
-                                      BaseOrganizationUserAdmin)
+from organizations.base_admin import (
+    BaseOrganizationAdmin,
+    BaseOrganizationOwnerAdmin,
+    BaseOrganizationUserAdmin,
+)
+from phonenumber_field.formfields import PhoneNumberField
+from swapper import load_model
 
 from . import settings as app_settings
-from .base import BaseAdmin
-from .models import (Group, Organization, OrganizationOwner, OrganizationUser,
-                     User)
 from .multitenancy import MultitenantAdminMixin
+from .utils import BaseAdmin
 
+Group = load_model('openwisp_users', 'Group')
+Organization = load_model('openwisp_users', 'Organization')
+OrganizationOwner = load_model('openwisp_users', 'OrganizationOwner')
+OrganizationUser = load_model('openwisp_users', 'OrganizationUser')
+User = get_user_model()
 logger = logging.getLogger(__name__)
 
 
@@ -42,11 +58,18 @@ class EmailAddressInline(admin.StackedInline):
         """
         return False
 
+    def has_change_permission(self, request, obj=None):
+        if user_not_allowed_to_change_owner(request.user, obj):
+            self.can_delete = False
+            return False
+        return super().has_change_permission(request, obj)
+
 
 class RequiredInlineFormSet(BaseInlineFormSet):
     """
     Generates an inline formset that is required
     """
+
     def _construct_form(self, i, **kwargs):
         """
         Override the method to change the form attribute empty_permitted
@@ -57,6 +80,16 @@ class RequiredInlineFormSet(BaseInlineFormSet):
         return form
 
 
+class OrganizationOwnerInline(admin.StackedInline):
+    model = OrganizationOwner
+    extra = 0
+
+    def has_change_permission(self, request, obj=None):
+        if obj and not request.user.is_superuser and not request.user.is_owner(obj):
+            return False
+        return super().has_change_permission(request, obj)
+
+
 class OrganizationUserInline(admin.StackedInline):
     model = OrganizationUser
     formset = RequiredInlineFormSet
@@ -72,14 +105,19 @@ class OrganizationUserInline(admin.StackedInline):
         if request.user.is_superuser:
             return formset
         if not request.user.is_superuser and not obj:
-            operator_orgs = OrganizationUser.objects.values_list('organization') \
-                                                    .filter(user=request.user,
-                                                            is_admin=True)
-            formset.form.base_fields['organization'].queryset = \
-                Organization.objects.filter(pk__in=operator_orgs)
+            operator_orgs = []
+            for pk, value in request.user.organizations_dict.items():
+                if value['is_admin']:
+                    operator_orgs.append(pk)
+            formset.form.base_fields[
+                'organization'
+            ].queryset = Organization.objects.filter(pk__in=operator_orgs)
         if not request.user.is_superuser and obj:
-            formset.form.base_fields['organization'].queryset = \
-                Organization.objects.filter(pk__in=request.user.organizations_pk)
+            formset.form.base_fields[
+                'organization'
+            ].queryset = Organization.objects.filter(
+                pk__in=request.user.organizations_dict.keys()
+            )
         return formset
 
     def get_extra(self, request, obj=None, **kwargs):
@@ -99,34 +137,69 @@ class OrganizationUserInlineReadOnly(OrganizationUserInline):
     def has_add_permission(self, request, obj=None):
         return False
 
+    def has_change_permission(self, request, obj=None):
+        if user_not_allowed_to_change_owner(request.user, obj):
+            return False
+        return super().has_change_permission(request, obj)
 
-class EmailRequiredMixin(forms.ModelForm):
+
+class UserFormMixin(forms.ModelForm):
     email = forms.EmailField(label=_('Email'), max_length=254, required=True)
 
+    def validate_user_groups(self, data):
+        is_staff = data.get('is_staff')
+        is_superuser = data.get('is_superuser')
+        groups = data.get('groups')
+        if is_staff and not is_superuser and not groups:
+            raise ValidationError(
+                {'groups': _('A staff user must belong to a group, please select one.')}
+            )
+
+    def clean(self):
+        cleaned_data = super().clean()
+        self.validate_user_groups(cleaned_data)
+        return cleaned_data
+
+
+class UserCreationForm(UserFormMixin, BaseUserCreationForm):
+    phone_number = PhoneNumberField(widget=forms.TextInput(), required=False)
 
-class UserCreationForm(EmailRequiredMixin, BaseUserCreationForm):
     class Meta(BaseUserCreationForm.Meta):
-        fields = ['username', 'email', 'password1', 'password2', 'is_staff']
-        fields_superuser = fields[:] + ['is_superuser']
+        fields = [
+            'username',
+            'email',
+            'password1',
+            'password2',
+        ]
+        personal_fields = ['first_name', 'last_name', 'phone_number']
         fieldsets = (
-            (None, {
-                'classes': ('wide',),
-                'fields': fields,
-            }),
+            (None, {'classes': ('wide',), 'fields': fields}),
+            ('Personal Info', {'classes': ('wide',), 'fields': personal_fields}),
+            (
+                'Permissions',
+                {'classes': ('wide',), 'fields': ['is_active', 'is_staff', 'groups']},
+            ),
         )
         fieldsets_superuser = (
-            (None, {
-                'classes': ('wide',),
-                'fields': fields_superuser,
-            }),
+            (None, {'classes': ('wide',), 'fields': fields}),
+            ('Personal Info', {'classes': ('wide',), 'fields': personal_fields}),
+            (
+                'Permissions',
+                {
+                    'classes': ('wide',),
+                    'fields': ['is_active', 'is_staff', 'is_superuser', 'groups'],
+                },
+            ),
         )
 
     class Media:
-        js = ('admin/js/jquery.init.js',
-              'openwisp-users/js/addform.js',)
+        js = (
+            'admin/js/jquery.init.js',
+            'openwisp-users/js/addform.js',
+        )
 
 
-class UserChangeForm(EmailRequiredMixin, BaseUserChangeForm):
+class UserChangeForm(UserFormMixin, BaseUserChangeForm):
     pass
 
 
@@ -135,33 +208,45 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
     form = UserChangeForm
     ordering = ['-date_joined']
     readonly_fields = ['last_login', 'date_joined']
-    list_display = ['username',
-                    'email',
-                    'is_active',
-                    'is_staff',
-                    'is_superuser',
-                    'date_joined',
-                    'last_login']
+    list_display = [
+        'username',
+        'email',
+        'is_active',
+        'is_staff',
+        'is_superuser',
+        'date_joined',
+        'last_login',
+    ]
     inlines = [EmailAddressInline, OrganizationUserInline]
     save_on_top = True
-    actions = ['make_inactive', 'make_active']
+    actions = ['delete_selected_overridden', 'make_inactive', 'make_active']
+
+    # To ensure extended apps use this template.
+    change_form_template = 'admin/openwisp_users/user/change_form.html'
 
     def require_confirmation(func):
         """
         Decorator to lead to a confirmation page.
-        This has been used rather than simply adding the same lines in action functions
-        inorder to avoid repetition of the same lines in the two added actions and more actions
+        This has been used rather than simply adding the same lines
+        in action functions inorder to avoid repetition of the same
+        lines in the two added actions and more actions
         incase they are added in future.
         """
+
         def wrapper(modeladmin, request, queryset):
             opts = modeladmin.model._meta
             if request.POST.get('confirmation') is None:
                 request.current_app = modeladmin.admin_site.name
-                context = {'action': request.POST['action'],
-                           'queryset': queryset,
-                           'opts': opts}
-                return TemplateResponse(request, 'admin/action_confirmation.html', context)
+                context = {
+                    'action': request.POST['action'],
+                    'queryset': queryset,
+                    'opts': opts,
+                }
+                return TemplateResponse(
+                    request, 'admin/action_confirmation.html', context
+                )
             return func(modeladmin, request, queryset)
+
         wrapper.__name__ = func.__name__
         return wrapper
 
@@ -170,9 +255,15 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
         queryset.update(is_active=False)
         count = queryset.count()
         if count:
-            self.message_user(request,
-                              _(f'Successfully made {count} {model_ngettext(self.opts, count)} inactive.'),
-                              messages.SUCCESS)
+            self.message_user(
+                request,
+                _(
+                    f'Successfully made {count} '
+                    f'{model_ngettext(self.opts, count)} inactive.'
+                ),
+                messages.SUCCESS,
+            )
+
     make_inactive.short_description = _('Flag selected users as inactive')
 
     @require_confirmation
@@ -180,9 +271,15 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
         queryset.update(is_active=True)
         count = queryset.count()
         if count:
-            self.message_user(request,
-                              _(f'Successfully made {count} {model_ngettext(self.opts, count)} active.'),
-                              messages.SUCCESS)
+            self.message_user(
+                request,
+                _(
+                    f'Successfully made {count} '
+                    f'{model_ngettext(self.opts, count)} active.'
+                ),
+                messages.SUCCESS,
+            )
+
     make_active.short_description = _('Flag selected users as active')
 
     def get_list_display(self, request):
@@ -190,8 +287,7 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
         Hide `is_superuser` from column from operators
         """
         default_list_display = super().get_list_display(request)
-        if (not request.user.is_superuser and
-                'is_superuser' in default_list_display):
+        if not request.user.is_superuser and 'is_superuser' in default_list_display:
             # avoid editing the default_list_display
             operators_list_display = default_list_display[:]
             operators_list_display.remove('is_superuser')
@@ -200,8 +296,7 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
 
     def get_list_filter(self, request):
         filters = super().get_list_filter(request)
-        if (not request.user.is_superuser and
-                'is_superuser' in filters):
+        if not request.user.is_superuser and 'is_superuser' in filters:
             # hide is_superuser filter for non-superusers
             operators_filter_list = list(filters)
             operators_filter_list.remove('is_superuser')
@@ -237,18 +332,81 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
         return fields
 
     def has_change_permission(self, request, obj=None):
+        if user_not_allowed_to_change_owner(request.user, obj):
+            return False
         # do not allow operators to edit details of superusers
         # returns 403 if trying to access the change form of a superuser
-        if obj and obj.is_superuser and not request.user.is_superuser:  # pragma: no cover
+        if (
+            obj and obj.is_superuser and not request.user.is_superuser
+        ):  # pragma: no cover
             return False
         return super().has_change_permission(request, obj)
 
+    def has_delete_permission(self, request, obj=None):
+        if user_not_allowed_to_change_owner(request.user, obj):
+            return False
+        return super().has_delete_permission(request, obj)
+
+    def get_actions(self, request):
+        actions = super().get_actions(request)
+        if not request.POST.get('post') and 'delete_selected' in actions:
+            del actions['delete_selected']
+        return actions
+
+    def delete_selected_overridden(self, request, queryset):
+        if not request.user.is_superuser:
+            users_pk = queryset.values_list('pk', flat=True)
+            owners_list = list(
+                OrganizationOwner.objects.filter(organization_user__user__in=users_pk)
+                .select_related('organization_user__user')
+                .values_list('organization_user__user__username', flat=True)
+            )
+            owners = ', '.join(owners_list)
+            excluded_owners_qs = queryset.exclude(username__in=owners_list)
+            # if trying to delete any owner, show an error message
+            count = len(owners_list)
+            if count:
+                self.message_user(
+                    request,
+                    ngettext(
+                        f"Can't delete %d organization owner: {owners}",
+                        f"Can't delete %d organization owners: {owners}",
+                        count,
+                    )
+                    % count,
+                    messages.ERROR,
+                )
+            # if trying to delete only owners, stop here
+            if queryset.exists() and not excluded_owners_qs.exists():
+                redirect_url = reverse(
+                    f'admin:{self.model._meta.app_label}_user_changelist'
+                )
+                return HttpResponseRedirect(redirect_url)
+            # otherwise proceed but remove owners from the delete queryset
+            else:
+                queryset = excluded_owners_qs
+        return delete_selected(self, request, queryset)
+
+    delete_selected_overridden.short_description = delete_selected.short_description
+
     def get_inline_instances(self, request, obj=None):
         """
-        Avoid displaying inline objects when adding a new user
+        1. Avoid displaying inline objects when adding a new user
+        2. Make OrganizationUserInline readonly for non-superuser
         """
         if obj:
-            return super().get_inline_instances(request, obj)
+            inlines = super().get_inline_instances(request, obj).copy()
+            if not request.user.is_superuser:
+                for inline in inlines:
+                    if isinstance(inline, OrganizationUserInline):
+                        orguser_index = inlines.index(inline)
+                        inlines.remove(inline)
+                        orguser_readonly = OrganizationUserInlineReadOnly(
+                            self.model, self.admin_site
+                        )
+                        inlines.insert(orguser_index, orguser_readonly)
+                        break
+            return inlines
         inline = OrganizationUserInline(self.model, self.admin_site)
         if request:
             if hasattr(inline, '_has_add_permission'):
@@ -260,8 +418,11 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
         return []
 
     def change_view(self, request, object_id, form_url='', extra_context=None):
-        if not request.user.is_superuser:
-            self.inlines[1] = OrganizationUserInlineReadOnly
+        extra_context = extra_context or {}
+        obj = self.model.objects.get(pk=object_id)
+        if user_not_allowed_to_change_owner(request.user, obj):
+            show_owner_warning = True
+            extra_context.update({'show_owner_warning': show_owner_warning})
         return super().change_view(request, object_id, form_url, extra_context)
 
     def save_model(self, request, obj, form, change):
@@ -272,11 +433,9 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
         super().save_model(request, obj, form, change)
         if obj.email:
             try:
-                EmailAddress.objects.add_email(request,
-                                               user=obj,
-                                               email=obj.email,
-                                               confirm=True,
-                                               signup=True)
+                EmailAddress.objects.add_email(
+                    request, user=obj, email=obj.email, confirm=True, signup=True
+                )
             except Exception as e:
                 logger.exception(
                     'Got exception {} while sending '
@@ -289,8 +448,12 @@ class UserAdmin(MultitenantAdminMixin, BaseUserAdmin, BaseAdmin):
 base_fields = list(UserAdmin.fieldsets[1][1]['fields'])
 additional_fields = ['bio', 'url', 'company', 'location', 'phone_number']
 UserAdmin.fieldsets[1][1]['fields'] = base_fields + additional_fields
-UserAdmin.add_fieldsets[0][1]['fields'] = ('username', 'email',
-                                           'password1', 'password2')
+UserAdmin.add_fieldsets[0][1]['fields'] = (
+    'username',
+    'email',
+    'password1',
+    'password2',
+)
 UserAdmin.search_fields += ('phone_number',)
 
 
@@ -298,31 +461,41 @@ class GroupAdmin(BaseGroupAdmin, BaseAdmin):
     pass
 
 
-class OrganizationAdmin(BaseOrganizationAdmin, BaseAdmin, UUIDAdmin):
+class OrganizationAdmin(
+    MultitenantAdminMixin, BaseOrganizationAdmin, BaseAdmin, UUIDAdmin
+):
     view_on_site = False
-    inlines = []
+    inlines = [OrganizationOwnerInline]
     readonly_fields = ['uuid']
     ordering = ['name']
 
+    def get_inline_instances(self, request, obj=None):
+        """
+        Remove OrganizationOwnerInline from organization add form
+        """
+        inlines = super().get_inline_instances(request, obj).copy()
+        if not obj:
+            for inline in inlines:
+                if isinstance(inline, OrganizationOwnerInline):
+                    inlines.remove(inline)
+                    break
+        return inlines
+
     def has_change_permission(self, request, obj=None):
         """
-        Allow operator to change an organization only if
-        they is an admin of that organization
+        Allow only managers and superuser to change organization
         """
-        if obj and not request.user.is_superuser:
-            try:
-                org = OrganizationUser.objects.get(organization=obj, user=request.user)
-                if not org.is_admin:
-                    return False
-            except OrganizationUser.DoesNotExist:
-                pass
+        if obj and not request.user.is_superuser and not request.user.is_manager(obj):
+            return False
         return super().has_change_permission(request, obj)
 
     class Media(UUIDAdmin.Media):
         css = {'all': ('openwisp-users/css/admin.css',)}
 
 
-class OrganizationUserAdmin(MultitenantAdminMixin, BaseOrganizationUserAdmin, BaseAdmin):
+class OrganizationUserAdmin(
+    MultitenantAdminMixin, BaseOrganizationUserAdmin, BaseAdmin
+):
     view_on_site = False
 
     def get_readonly_fields(self, request, obj=None):
@@ -339,16 +512,16 @@ class OrganizationUserAdmin(MultitenantAdminMixin, BaseOrganizationUserAdmin, Ba
         operators should not delete organization users of organizations
         where they are not admins
         """
-        if obj and not request.user.is_superuser:
-            operator_org = OrganizationUser.objects.get(organization=obj.organization,
-                                                        user=request.user)
-            if operator_org.is_admin:
-                return True
-            else:
-                return False
+        if request.user.is_superuser:
+            return True
+        if obj and not request.user.is_manager(obj.organization):
+            return False
+        return super().has_delete_permission(request, obj)
 
 
-class OrganizationOwnerAdmin(MultitenantAdminMixin, BaseOrganizationOwnerAdmin, BaseAdmin,):
+class OrganizationOwnerAdmin(
+    MultitenantAdminMixin, BaseOrganizationOwnerAdmin, BaseAdmin,
+):
     list_display = ('get_user', 'organization')
 
     def get_user(self, obj):
@@ -373,8 +546,29 @@ admin.site.register(Group, GroupAdmin)
 
 # unregister some admin components to keep the admin interface simple
 # we can re-enable these models later when they will be really needed
-for model in [('account', 'EmailAddress'),
-              ('socialaccount', 'SocialApp'),
-              ('socialaccount', 'SocialToken'),
-              ('socialaccount', 'SocialAccount')]:
-    admin.site.unregister(apps.get_model(*model))
+admin.site.unregister(apps.get_model('account', 'EmailAddress'))
+if allauth_settings.SOCIALACCOUNT_ENABLED:
+    for model in [
+        ('socialaccount', 'SocialApp'),
+        ('socialaccount', 'SocialToken'),
+        ('socialaccount', 'SocialAccount'),
+    ]:
+        admin.site.unregister(apps.get_model(*model))
+
+
+if 'rest_framework.authtoken' in settings.INSTALLED_APPS and not settings.DEBUG:
+    Token = apps.get_model('authtoken', 'Token')
+
+    try:
+        admin.site.unregister(Token)
+    except NotRegistered:
+        pass
+
+
+def user_not_allowed_to_change_owner(user, obj):
+    return (
+        obj
+        and not user.is_superuser
+        and user.pk != obj.pk
+        and obj.is_owner_of_any_organization
+    )
